' ========================================================================================
' GdipWindingModeOutline
' ========================================================================================

' SED_PBWIN
#COMPILE EXE
#DIM ALL
#INCLUDE "GDIPLUS.INC"

' ========================================================================================
' The following example creates a GraphicsPath object and calls the GdiAddPathClosedCurve
' method to add a closed cardinal spline to the path. The code calls the GdipWidenPath
' function to widen the path and then draws the path. Next, the code calls the
' GdipWindingModeOutline function The code calls the GdipTranslateWorldTransform function
' so that the outlined path drawn by the subsequent call to GdipDrawPath sits to the right
' of the first path.
' ========================================================================================
SUB GDIP_WindingModeOutline (BYVAL hdc AS DWORD)

   LOCAL hStatus AS LONG
   LOCAL pGraphics AS DWORD
   LOCAL pPath AS DWORD
   LOCAL pBluePen AS DWORD
   LOCAL pGreenPen AS DWORD
   DIM   pts(3) AS POINTF
   LOCAL result AS LONG
   LOCAL i AS LONG

   hStatus = GdipCreateFromHDC(hdc, pGraphics)

   hStatus = GdipCreatePen1(GDIP_ARGB(255, 0, 0, 255), 1, %UnitWorld, pBluePen)
   hStatus = GdipCreatePen1(GDIP_ARGB(255, 255, 255, 0), 10, %UnitWorld, pGreenPen)

   hStatus = GdipCreatePath(%FillModeAlternate, pPath)

   pts(0).x = 20  : pts(0).y = 20
   pts(1).x = 160 : pts(1).y = 100
   pts(2).x = 140 : pts(2).y = 60
   pts(3).x = 60  : pts(3).y = 100

   hStatus = GdipAddPathClosedCurve(pPath, pts(0), 4)

   hStatus = GdipWidenPath(pPath, pGreenPen, %NULL, FlatnessDefault)
   hStatus = GdipDrawPath(pGraphics, pBluePen, pPath)
   hStatus = GdipWindingModeOutline(pPath, %NULL, FlatnessDefault)
   hStatus = GdipTranslateWorldTransform(pGraphics, 180, 0, %MatrixOrderPrepend)
   hStatus = GdipDrawPath(pGraphics, pBluePen, pPath)

   ' // Cleanup
   IF pBluePen THEN GdipDeletePen(pBluePen)
   IF pGreenPen THEN GdipDeletePen(pGreenPen)
   IF pPath THEN GdipDeletePath(pPath)
   IF pGraphics THEN GdipDeleteGraphics(pGraphics)

END SUB
' ========================================================================================

' ========================================================================================
' Main
' ========================================================================================
FUNCTION WINMAIN (BYVAL hInstance AS DWORD, BYVAL hPrevInstance AS DWORD, BYVAL lpszCmdLine AS ASCIIZ PTR, BYVAL nCmdShow AS LONG) AS LONG

   LOCAL hr AS LONG
   LOCAL hDlg AS DWORD
   LOCAL token AS DWORD
   LOCAL StartupInput AS GdiplusStartupInput

   ' Initialize GDI+
   StartupInput.GdiplusVersion = 1
   hr = GdiplusStartup(token, StartupInput, BYVAL %NULL)
   IF hr THEN
      MSGBOX "Error initializing GDI+"
      EXIT FUNCTION
   END IF

   DIALOG NEW 0, "GdipWindingModeOutline", , , 280, 170, %WS_OVERLAPPED OR %WS_THICKFRAME OR %WS_SYSMENU OR _
   %WS_MINIMIZEBOX OR %WS_MAXIMIZEBOX OR %WS_VISIBLE OR %DS_CENTER TO hDlg
   DIALOG SET COLOR hDlg, %BLACK, %WHITE
   DIALOG SHOW MODAL hDlg, CALL DlgProc

   ' Shutdown GDI+
   GdiplusShutdown token

END FUNCTION
' ========================================================================================

' ========================================================================================
' Main Dialog procedure
' ========================================================================================
CALLBACK FUNCTION DlgProc() AS LONG

   LOCAL hDC AS DWORD
   LOCAL ps AS PAINTSTRUCT

   SELECT CASE CBMSG

      CASE %WM_COMMAND
         SELECT CASE CBCTL
            CASE %IDCANCEL
               IF CBCTLMSG = %BN_CLICKED THEN DIALOG END CBHNDL, 0
         END SELECT

      CASE %WM_PAINT
         hDC = BeginPaint(CBHNDL, ps)
         GDIP_WindingModeOutline hDC
         EndPaint(CBHNDL, ps)

   END SELECT

END FUNCTION
' ========================================================================================
